@@ -20,7 +20,8 @@ struct bw {
 	OPTIONS	o;
 	void	*object;
 
-	int	linums;
+	int	lincols;	/* Line number column width */
+	int	curlin;		/* Cursor line (highlighted) */
 	int	top_changed;	/* Top changed */
 	struct lattr_db *db;	/* line attribute database */
 	int	shell_flag;	/* Cursor should follow shell cursor in this window */
@@ -31,12 +32,19 @@ extern int opt_mid;	/* Controls how window scrolls: when set, scroll window enou
 extern int opt_left;
 extern int opt_right;
 
+extern int selectmask;	/* Attribute mask for selected text */
+extern int selectatr;	/* Attribute value for selected text */
+extern int bg_linum;	/* Attribute value for line numbers */
+extern int bg_curlinum; /* Attribute value for the line number of the current line */
+extern int bg_curlin;	/* Attribute value for the current line */
+extern int curlinmask;	/* Mask for the current line */
+
 void bwfllw(W *w);
 void bwfllwt(W *w);
 void bwfllwh(W *w);
 void bwins(BW *w, off_t l, off_t n, int flg);
 void bwdel(BW *w, off_t l, off_t n, int flg);
-void bwgen(BW *w, int linums);
+void bwgen(BW *w, int linums, int linchg);
 void bwgenh(BW *w);
 BW *bwmk(W *window, B *b, int prompt);
 void bwmove(BW *w, ptrdiff_t x, ptrdiff_t y);
@@ -46,6 +54,7 @@ int ustat(W *w, int k);
 int ucrawll(W *w, int k);
 int ucrawlr(W *w, int k);
 void orphit(BW *bw);
+int calclincols(BW *bw);
 
 extern int marking;	/* Anchored block marking mode */
 
