@@ -1455,42 +1455,7 @@ char **get_encodings()
 	}
 
 	/* External maps */
-
-	p = getenv("HOME");
-	if (p) {
-		joe_snprintf_1(buf,SIZEOF(buf),"%s/.joe/charmaps",p);
-		if (!chpwd(buf) && (t = rexpnd("*"))) {
-			for (x = 0; x != aLEN(t); ++x)
-				if (zcmp(t[x],"..")) {
-					for (y = 0; y != aLEN(encodings); ++y)
-						if (!zcmp(t[x],encodings[y]))
-							break;
-					if (y == aLEN(encodings)) {
-						r = vsncpy(NULL,0,sv(t[x]));
-						encodings = vaadd(encodings,r);
-					}
-				}
-			varm(t);
-		}
-	}
-
-	if (!chpwd((JOEDATA "charmaps")) && (t = rexpnd("*"))) {
-		for (x = 0; x != aLEN(t); ++x)
-			if (zcmp(t[x],"..")) {
-				for (y = 0; y != aLEN(encodings); ++y)
-					if (!zcmp(t[x],encodings[y]))
-						break;
-				if (y == aLEN(encodings)) {
-					r = vsncpy(NULL,0,sv(t[x]));
-					encodings = vaadd(encodings,r);
-				}
-			}
-		varm(t);
-	}
-
-	chpwd(oldpwd);
-
-	return encodings;
+	return find_configs(encodings, NULL, "charmaps", "charmaps");
 }
 
 /* This is not correct... (EBCDIC for example) */
