@@ -22,6 +22,7 @@ int stack_count = 0;
 static int state_count = 0; /* Max transitions possible without cycling */
 
 struct high_syntax *ansi_syntax;
+struct high_syntax *syntax_list;
 
 /* ANSI highlighter */
 
@@ -35,6 +36,7 @@ static HIGHLIGHT_STATE ansi_parse(P *line, HIGHLIGHT_STATE h_state)
 	int *attr = attr_buf;
 	int *attr_end = attr_buf + attr_size;
 	int c;
+	int bold = 0; /* Save bold state for extended scheme colors */
 
 	int state = IDLE; /* h_state.saved_s[0]; */
 	int accu = 0; /* h_state.saved_s[1]; */
@@ -93,8 +95,10 @@ static HIGHLIGHT_STATE ansi_parse(P *line, HIGHLIGHT_STATE h_state)
 				if (c == ';' || c == 'm') {
 					if (accu == 0) {
 						current_attr = 0;
+						bold = 0;
 					} else if (accu == 1) {
 						current_attr |= BOLD;
+						bold = 1;
 					} else if (accu == 4) {
 						current_attr |= UNDERLINE;
 					} else if (accu == 5) {
@@ -102,9 +106,22 @@ static HIGHLIGHT_STATE ansi_parse(P *line, HIGHLIGHT_STATE h_state)
 					} else if (accu == 7) {
 						current_attr |= INVERSE;
 					} else if (accu >= 30 && accu <= 37) {
-						current_attr = ((current_attr & ~FG_MASK) | FG_NOT_DEFAULT | ((accu - 30) << FG_SHIFT));
+						if (bold && curschemeset && curschemeset->termcolors[accu - 22].type != COLORSPEC_TYPE_NONE) {
+							/* Remapped extended color */
+							current_attr = (current_attr & ~FG_MASK) | (curschemeset->termcolors[accu - 22].atr & FG_MASK & ~BOLD);
+						} else if (curschemeset && curschemeset->termcolors[accu - 30].type != COLORSPEC_TYPE_NONE) {
+							/* Remapped by scheme */
+							current_attr = (current_attr & ~FG_MASK) | (curschemeset->termcolors[accu - 30].atr & FG_MASK);
+						} else {
+							current_attr = (current_attr & ~FG_MASK) | FG_NOT_DEFAULT | ((accu - 30) << FG_SHIFT) | (-bold & BOLD);
+						}
 					} else if (accu >= 40 && accu <= 47) {
-						current_attr = ((current_attr & ~BG_MASK) | BG_NOT_DEFAULT | ((accu - 40) << BG_SHIFT));
+						if (curschemeset && curschemeset->termcolors[accu - 40].type != COLORSPEC_TYPE_NONE) {
+							/* Remapped by scheme */
+							current_attr = (current_attr & ~BG_MASK) | (((curschemeset->termcolors[accu - 40].atr & FG_MASK) >> FG_SHIFT) << BG_SHIFT);
+						} else {
+							current_attr = (current_attr & ~BG_MASK) | BG_NOT_DEFAULT | ((accu - 40) << BG_SHIFT);
+						}
 					}
 					if (c == ';') {
 						accu = 0;
@@ -377,11 +394,12 @@ static struct high_state *find_state(struct high_syntax *syntax,char *name)
 	/* It doesn't exist, so create it */
 	if(!state) {
 		state=(struct high_state *)joe_malloc(SIZEOF(struct high_state));
-		state->name=zdup(name);
-		state->no=syntax->nstates;
-		state->color=FG_WHITE;
+		state->name = zdup(name);
+		state->no = syntax->nstates;
+		state->color = 0;
+		state->colorp = NULL;
 		/* Expand the state table if necessary */
-		if(syntax->nstates==syntax->szstates)
+		if (syntax->nstates==syntax->szstates)
 			syntax->states=(struct high_state **)joe_realloc(syntax->states,SIZEOF(struct high_state *)*(syntax->szstates*=2));
 		syntax->states[syntax->nstates++]=state;
 		rtree_init(&state->rtree);
@@ -437,14 +455,10 @@ static struct high_cmd *mkcmd()
 	return cmd;
 }
 
-/* Globally defined colors */
-
-struct high_color *global_colors;
-
-static struct high_color *find_color(struct high_color *colors,char *name,char *syn)
+static struct color_def *find_color(struct color_def *colors,char *name,char *syn)
 {
 	char bf[256];
-	struct high_color *color;
+	struct color_def *color;
 	joe_snprintf_2(bf, SIZEOF(bf), "%s.%s", syn, name);
 	for (color = colors; color; color = color->next)
 		if (!zcmp(color->name,bf)) break;
@@ -455,43 +469,37 @@ static struct high_color *find_color(struct high_color *colors,char *name,char *
 	return color;
 }
 
-void parse_color_def(struct high_color **color_list,const char *p,char *name,int line)
+void parse_syntax_color_def(struct color_def **color_list,const char *p,char *name,int line)
 {
 	char bf[256];
 	if(!parse_tows(&p, bf)) {
-		struct high_color *color, *gcolor;
+		struct color_def *color, *gcolor;
 
 		/* Find color */
 		color=find_color(*color_list,bf,name);
 
 		/* If it doesn't exist, create it */
 		if(!color) {
-			color = (struct high_color *)joe_malloc(SIZEOF(struct high_color));
-			color->name = zdup(bf);
-			color->color = 0;
+			color = (struct color_def *)joe_malloc(SIZEOF(struct color_def));
+			color->name = atom_add(bf);
 			color->next = *color_list;
 			*color_list = color;
 		} else {
 			logerror_2(joe_gettext(_("%s %d: Class already defined\n")),name,line);
 		}
 
-		/* Find it in global list */
-		if (color_list != &global_colors && (gcolor=find_color(global_colors,bf,name))) {
-			color->color = gcolor->color;
-		} else {
-			/* Parse color definition */
-			while(parse_ws(&p,'#'), !parse_ident(&p,bf,SIZEOF(bf))) {
-				color->color |= meta_color(bf);
-			}
-		}
+		parse_color_def(&p, color);
+
+		/* Older syntaxes had colors specified with color classes.  For backwards
+		   compatibility, we'll try to map those into the scheme via the term_colors.
+		   We preserve the parsed color_spec here because the scheme/syntax color
+		   resolution procedure will overwrite color->spec. */
+		memcpy(&color->orig, &color->spec, SIZEOF(struct color_spec));
 	} else {
 		logerror_2(joe_gettext(_("%s %d: Missing class name\n")),name,line);
 	}
 }
 
-/* Load syntax file */
-
-struct high_syntax *syntax_list;
 
 /* Dump sytnax file */
 
@@ -810,7 +818,7 @@ static struct high_state *load_dfa(struct high_syntax *syntax)
 			/* Ignore this line because of ifdef */
 		} else if(!parse_char(&p, '=')) {
 			/* Parse color */
-			parse_color_def(&syntax->color,p,name,line);
+			parse_syntax_color_def(&syntax->color,p,name,line);
 		} else if ((syntax->subr && !this_one) || (!syntax->subr && inside_subr)) {
 			/* Ignore this line because it's not the code we want */
 		} else if(!parse_char(&p, ':')) {
@@ -823,16 +831,16 @@ static struct high_state *load_dfa(struct high_syntax *syntax)
 
 				parse_ws(&p,'#');
 				if(!parse_tows(&p,bf)) {
-					struct high_color *color;
+					struct color_def *color;
 					for(color=syntax->color;color;color=color->next)
 						if(!zcmp(color->name,bf))
 							break;
-					if(color)
-						state->color=color->color;
-					else {
-						state->color=0;
+					if(!color) {
 						logerror_2(joe_gettext(_("%s %d: Unknown class\n")),name,line);
 					}
+
+					state->color = 0;
+					state->colorp = color;
 					while(parse_ws(&p, '#'), !parse_ident(&p, bf, SIZEOF(bf))) {
 						if(!zcmp(bf, "comment")) {
 							state->color |= CONTEXT_COMMENT;
@@ -951,6 +959,7 @@ struct high_syntax *load_syntax_subr(const char *name,char *subr,struct high_par
 
 	if (load_dfa(syntax)) {
 		/* dump_syntax(syntax); */
+		resolve_syntax_colors(curschemeset, syntax);
 		return syntax;
 	} else {
 		if(syntax_list == syntax)
